package CheckReleaseEditor::Plugin;

use strict;
use warnings;

use MT;
use base qw(MT::Plugin);
use CheckRelease::Plugin;
use CheckReleaseEditor::L10N;

my $plugin = CheckReleaseEditor::Plugin->new({
	id => 'CheckReleaseEditor',
	key => __PACKAGE__,
	name => 'CheckReleaseEditor',
	l10n_class => 'CheckReleaseEditor::L10N',
	description => '<__trans phrase="Support your checking entries." />',
	version => $CheckRelease::Plugin::VERSION,
});

sub init_registry {
	my ($plugin) = @_;
	$plugin->registry({
		callbacks => {
			'cms_pre_save.entry' => {
				code => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_cms_pre_save_entry',
				priority => 1
			},
			'cms_post_save.entry' => {
				priority => 9, # avoid conflict with TimeLimit
				code => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_cms_post_save',
			},
			'template_param.edit_entry' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_template_param_edit_entry',
			'template_source.entry_table' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_template_source_entry_table',
			'template_param.entry_table' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_template_param_entry_table',
			'template_source.edit_entry' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_template_source_edit_entry',
			'cms_pre_preview' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_cms_pre_preview',
			'template_param.list_entry' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_template_param_list_entry',
			'cms_save_filter.entry' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_cms_save_filter_entry',
			'template_source.list_entry' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_template_source_list_entry',
			'pre_build' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_pre_build',
			# over MT5.1
			'template_source.entry_list_header' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_template_source_list_entry',
			'template_param.list_common' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_template_param_list_entry',
			'cms_post_bulk_save.entries' => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_cms_post_save_bulk_save_entries',
		},
		applications => {
			cms => {
				callbacks => {
					pre_run => '$CheckReleaseEditor::CheckReleaseEditor::CMS::Entry::_hdlr_pre_run',
				},
			},
		},
		tags => {
			function => {
				ReturnAuthorDisplayName => '$CheckReleaseEditor::CheckReleaseEditor::Template::Tags::Author::_hdlr_return_author_display_name',
				ReturnAuthorID => '$CheckReleaseEditor::CheckReleaseEditor::Template::Tags::Author::_hdlr_return_author_id',
				ReturnAuthorName => '$CheckReleaseEditor::CheckReleaseEditor::Template::Tags::Author::_hdlr_return_author_name',
			},
		},
	});
}

sub instance {
	$plugin;
}
1;
