package CheckReleaseWriter::CMS::Entry;

use strict;
use warnings;

use MT;
use base qw(MT::Plugin);

use MT::App::CMS;
use MT::Revisable;
use MT::Util qw(epoch2ts format_ts is_valid_email encode_js);
use CheckReleaseWriter::Util qw( list_html_tmpl );
use Ideamans::Util qw(array_search);
use Data::Dumper;

use CheckRelease::CMS::Entry;
use CheckRelease::Util qw( get_permissions get_config_value get_config_hash
		check_url _scope want_cross_check get_checkers is_editor mail_head is_administer
		update_future_que load_latest_revision send_notice_mail cf_meta load_checkers
		get_crChangeBtnLabel unable_plugin set_param_session );

use constant SUBSCRIBED => CheckRelease::CMS::Entry::SUBSCRIBED();
use constant APPROVAL => CheckRelease::CMS::Entry::APPROVAL();
use constant RETURN => CheckRelease::CMS::Entry::RETURN();

sub _hdlr_template_source_edit_entry {
	my ($eh, $app, $tmpl) = @_;
	my $q = $app->param;
	return unless $q->param('_type') eq 'entry';
	return $$tmpl if unable_plugin();

	my $user = $app->user;
	my $blog_id = $app->param('blog_id') || 0;

	# 承認者なら処理不要
	return if is_administer($user, $blog_id);

	my $id = $app->param('id') || '';
	if ( MT->version_number < 5.1 ) {

		# 承認必須でも下書きできるように「下書き」ボタンの追加
		my $blog_id = $app->param('blog_id');
		my $config = get_config_hash("blog:$blog_id");
		if ( !($config->{select_checker} && !$config->{check_required}) ) {
			my $old = q{(<div class="actions-bar.+?<button.+?class="preview.+?<\/div>)};
			my $save_draft_btn = q{<div class="actions-bar save-action" style="width:100%; clear:both;float:none;margin-top:12px;">
        <button
           name="save_draft"
           type="submit"
           title="<__trans phrase="Draft" />"
           class="preview action button btn-save-draft"
           onClick="javascript:jQuery('#save_draft').val(1);"
           style="float:right;">
          <__trans phrase="Draft" />
        </button>
      </div>
      <input type="hidden" name="save_draft" id="save_draft" value="0" />
      };
			$$tmpl =~ s!$old!$1$save_draft_btn!gms;
		}

		# 「下書き保存」の変更
		my $draft_button_text = '<mt:var\s*name="draft_button_text">';
		my $save_draft = '<__trans\s*phrase="Save Draft">';
		my $draft_button_html = q{<span id="cr_draft_button_text"><mt:setvarblock name="label_checked"><mt:unless name="select_checker">1</mt:unless><mt:if name="check_required">1</mt:if></mt:setvarblock><__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section></span><script type="text/javascript">jQuery("#cr_draft_button_text").parent("button").each(function()\{jQuery(this).attr("title", "<__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section>");\});</script>};
		$$tmpl =~ s!$draft_button_text!$draft_button_html!gms;
		$$tmpl =~ s!$save_draft!$draft_button_html!gms;

		return unless $id;

		# JavaScript内の「更新」の変更
		my $button_text_in_script = '(\'<mt:var\s*name="button_text">\')';
		my $draft_button_html_no_script = q{'<span id="cr_draft_button_text"><mt:setvarblock name="label_checked"><mt:unless name="select_checker">1</mt:unless><mt:if name="check_required">1</mt:if></mt:setvarblock><__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section></span>jQuery("#cr_draft_button_text").parent("button").each(function()\{jQuery(this).attr("title", "<__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section>");\});'};
		$$tmpl =~ s!$button_text_in_script!$draft_button_html_no_script!gms;

		# 「更新」の変更
		my $button_text = '<mt:var\s*name="button_text">';
		my $trans_update = '<__trans\s*phrase="Update">';
		$$tmpl =~ s!$button_text!$draft_button_html!gms;
		$$tmpl =~ s!$trans_update!$draft_button_html!gms;
	}
	else {
		# MT5.1 対応
		# 承認必須でも下書きできるように「下書き」ボタンの追加
		my $blog_id = $app->param('blog_id');
		my $config = get_config_hash("blog:$blog_id");
		if ( !($config->{select_checker} && !$config->{check_required}) ) {
			my $old = q{(<div class="preview\-action">.+?<\/div>)};
			my $save_draft_btn = q{<div class="save-action" style="width:100%;clear:both;float:none;padding-top:12px;">
        <button
           name="save_draft"
           type="submit"
           title="<__trans phrase="Draft" />"
           class="publish action button btn-save-draft"
           onClick="javascript:jQuery('#save_draft').val(1);">
          <__trans phrase="Draft" />
        </button>
      </div>
      };
			$$tmpl =~ s!$old!$1$save_draft_btn!gms;
		}

		# 「保存」の変更
		my $button_text = '<mt:var\s*name="button_text">';
		my $draft_button_html = q{<span id="cr_draft_button_text"><mt:setvarblock name="label_checked"><mt:unless name="select_checker">1</mt:unless><mt:if name="check_required">1</mt:if></mt:setvarblock><__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section></span>};
		my $draft_button_js = q{<script type="text/javascript">jQuery("#cr_draft_button_text").parent("button").each(function()\{jQuery(this).attr("title", "<__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section>");\});</script>};
		$$tmpl =~ s!$button_text!$draft_button_html$draft_button_js!gms;

		my $draft_button_text = q{jQuery\('button\.publish'\)\.text\('<mt:var\s*name="draft_button_text">'\);};
		my $draft_button_replace = "jQuery('button.publish').html('" .$draft_button_html ."');";
		$$tmpl =~ s!$draft_button_text!$draft_button_replace!gms;

		my $set_save_js = q{setSaveButtonText\(\);};
		my $cr_change_btn_label = q{crChangeBtnLabel();};
		$$tmpl =~ s!$set_save_js!$cr_change_btn_label!gms;
	}

	# ページロード時にボタンのラベルを切り替えるのJavaScriptの設置
	my $jquery_input_title = "jQuery\\('input#title'\\)";
	my $onload_changebtnlabel = get_crChangeBtnLabel('writer');
	$$tmpl =~ s!($jquery_input_title)!$onload_changebtnlabel\n$1!ms;


	# 差し戻し状態のhiddenの追加
	my $status_old = '';
	if ( MT->version_number < 5.1 ) {
		$status_old = '<input type="hidden" name="status" id="status"';
	}
	else {
		$status_old = '<div class="status">';
	}
	my $status_new = q{
<input type="hidden" name="save_draft" id="save_draft" value="0" />
<input type="hidden" name="check_status" id="check_status" value="<mt:var name="check_status" default="0" />" />
<input type="hidden" name="check_requested_authors" id="check_requested_authors" value="<mt:var name="check_requested_authors" />" />
<mt:if name="crr"><input type="hidden" name="crr" id="crr" value="<mt:var name="crr" />" /></mt:if>
<input type="hidden" name="r" id="r" value="<mt:var name="r" />" />
<mt:if name="crr"><input type="hidden" name="basename" value="<mt:var name="basename" />" /></mt:if>
};
	$$tmpl =~ s!$status_old!$status_new\n$status_old!msi;

	# リビジョンの新規作成の場合
	if ( $app->param('crr') ) {
		if ( ( defined($app->param('pre_r')) && $app->param('pre_r') == 0 )
			|| ( defined($app->param('r')) && $app->param('r') == 0 ) ) {
			# page_titleの修正
			my ($page_title_old,  $page_title_new);
			if ( MT->version_number < 5.1 ) {
				$page_title_old = '<\$mt:setvar name="page_title" value="<__trans phrase="Edit Entry">"\$>';
				$page_title_new = q{<$mt:setvar name="page_title" value="<__trans_section component="CheckRelease"><__trans phrase="Edit Entry: New Revision"></__trans_section>"$>};
			} else {
				$page_title_old = '<mt:setvar name="page_title" value="<__trans phrase="Edit Entry">">';
				$page_title_new = q{<$mt:setvar name="page_title" value="<__trans_section component="CheckRelease"><__trans phrase="Edit Entry: New Revision"></__trans_section>"$>};
			}
			# page_titleの修正
			$$tmpl =~ s!$page_title_old!$page_title_new!m;
		}

	}

  # リビジョン保存後に表示される「ブログ記事を見る」を非表示
  if ($app->param('crr') && $app->param('r')) {
    if($app->param('saved_added') || $app->param('saved_changes')) {
      $$tmpl =~ s!<\$mt:var\sname="view_link"\$>!!g;
    }
  }

}

sub _hdlr_template_param_edit_entry {
	my ($cb, $app, $param, $tmpl) = @_;
	return if unable_plugin();

	my $q = $app->param;
	return unless $q->param('_type') eq 'entry';
	my $user = $app->user;
	my $blog_id = $app->param('blog_id') || 0;

	# 投稿者が承認者の場合処理不要
	return if is_administer($user, $blog_id);

	$param->{crr} = $q->param('crr') || '';
	$param->{r} = defined($q->param('r')) ? $q->param('r') : '';

	# プレビューから戻った場合
	my ($rev_obj, $packed);
	if ( $app->param('reedit') ) {
		$param->{r} = $app->param('pre_r') if defined $app->param('pre_r');
		$param->{keywords} = $app->param('keywords') || '' if $app->param('crr');
		$param->{error} ||= $app->param('error') if $app->param('error');
	}
	else {
		# リビジョンを取得
		my $class = $app->param('_type') || 'entry';
		my $r = $app->param('r') || '';
		$rev_obj = load_latest_revision($app, $class, $app->param('id'));
		if ( $r ) {
			$packed = ref($rev_obj) ? $rev_obj->$class : {};

			require MT::Serialize;
			$packed = MT::Serialize->unserialize($packed) || {};

			# タグを復元
			$param->{tags} = $$packed->{tags} || ''
				unless $param->{r} eq '0';

			# アイテム情報の復元
			my $packed_objassets = defined($$packed->{objectasset})
				? $$packed->{objectasset}
				: [];
			if ( @$packed_objassets ) {
				my @ids = map { $_->{asset_id} } @$packed_objassets;
				require MT::Asset;
				my @assets = MT::Asset->load({ id => \@ids });
				my $assets = ();
				foreach my $asset (@assets) {
				    my $asset_1 = {
			            asset_id    => $asset->id,
			            asset_name  => $asset->file_name,
				        asset_type  => $asset->class
				    };
				    $asset_1->{asset_thumb} = $asset->thumbnail_url( Width => 100 )
				    	if $asset->class eq 'image';
				    push @{$assets}, $asset_1;
				}
				$param->{asset_loop} = $assets;
			}
		}
	}
	if ( defined($param->{r}) && $param->{r} ) {
		$param->{loaded_revision} = 1;
	}

	# リビジョンの新規作成の場合
	if ( $app->param('crr') ) {
		if ( $app->param('r') && $app->param('r') eq '0' ) {
			$param->{status_publish} = 0;
		}
	}

	# 公開ボタンのラベル表記「承認申請」or「保存」のパラメータ調整
	$param->{created_by_is_me} = 1;

	# 承認者に該当するユーザーを検索
	my $authors = get_checkers();

	my $plugin = $app->component('CheckRelease');

	# 公開ウィジェットのオブジェクトを取得
	my $publishing_node = $tmpl->getElementById('entry-publishing-widget');

	# 承認者選択かどうか
	$param->{select_checker} = get_config_value('select_checker') ? 1 : 0;
	$param->{check_required} = get_config_value('check_required') ? 1 : 0;

	# 承認者を選択しない場合は処理不要
	if ( get_config_value('select_checker') ) {

		# 承認者が存在しない場合はそのメッセージを表示
		unless ( @$authors ) {
			$param->{cr_message} = $plugin->translate("Can't find checkers.");
		}
		else {
			my $checkers = [];

			# プレビューから戻った場合は選択状態を復元
			if ( defined( $q->{param}->{'checkrelease_checkers[]'} ) ) {
				$checkers = $q->{param}->{'checkrelease_checkers[]'};
			}
			elsif ( ref($packed)
				&& exists($$packed->{check_requested_authors}) ) {
				# リビジョンの編集の場合承認者の設定を引き継ぎ
				@$checkers = split /\s*,\s*/, $$packed->{check_requested_authors}
					if defined($$packed->{check_requested_authors});
			}
			elsif ( my $entry_id = $q->param('id') ) {
				# 記事編集の場合
				my $entry = $app->model('entry')->load($entry_id);
				@$checkers = split /\s*,\s*/, $entry->check_requested_authors
					if ref($entry) && $entry->check_requested_authors;
			}
			my $object_loop = load_checkers( $checkers );
			$param->{cr_loop} = $object_loop;
		}

		# 承認者のHTML
		my $innerHTML = <<'EOT';
		<mt:if name="cr_message">
			<mt:var name="cr_message" />
		<mt:else>
			<p><__trans_section component="CheckRelease"><__trans phrase="You can select your checker."></__trans_section></p>
			<mt:loop name="cr_loop">
			<input type="checkbox" name="checkrelease_checkers[]" class="cr_check_item" value="<mt:var name="id" />"<mt:if name="checked"> checked="checked"</mt:if> onClick="javascript:crChangeBtnLabel();" /> <mt:var name="label" /><br />
			</mt:loop>
		</mt:if>
EOT
		$innerHTML .= '<script type="text/javascript">' .get_crChangeBtnLabel('writer') .'</script>';

		# 他のプラグインから承認者を任意の条件で変更できるようにコールバックを実行
		$app->run_callbacks('cr_template_param.edit_entry', $app, \$innerHTML, \$param)
			or die $app->errstr;

		# MTタグでHTMLを生成
		my $node_params;
		$node_params->{id} = 'checkrelease-checkeres';
		$node_params->{label} = $plugin->translate('Select Checkers');
		my $block = $tmpl->createElement('app:widget', $node_params);
		$block->innerHTML( $innerHTML );

		#公開ウィジェットの上に挿入
		$tmpl->insertBefore($block, $publishing_node);
	}

	# 差戻し理由のHTML
	if ( $app->param('id') && (!$app->param('r') || $app->param('r') != 0) ) {
		my $innerHTML2 = <<'EOD';
		<mt:if name="cr_display_note">
			<textarea id="check_note" name="check_note" class="text row"><mt:var name="check_note"/></textarea>
		<mt:else>
			<input type="hidden" id="check_note" name="check_note" value="<mt:var name="check_note" escape="html" />" />
			<mt:if name="check_note"><mt:var name="check_note" nl2br="1" /><mt:else><span style="color:#aaa;"><__trans_section component="CheckRelease"><__trans phrase="Nothing is noted."></__trans_section></span></mt:if>
		</mt:if>
EOD

		# 差戻し理由のフィールドを作成
		my $node_params;
		$node_params->{id} = 'checkrelease-returnto';
		$node_params->{label} = $plugin->translate('Returnto Reason');
		$param->{cr_display_note} = 0;

		my $block = $tmpl->createElement('app:widget', $node_params);
		$block->innerHTML( $innerHTML2 );
		$tmpl->insertBefore($block, $publishing_node);
	}

	1;
}

sub _hdlr_cms_pre_preview {
	my ($cb, $app, $entry, $data) = @_;
	return if unable_plugin();

	my $q = $app->param;
	return unless $q->param('_type') eq 'entry';
	my $user = $app->user;
	my $blog_id = $q->param('blog_id') || 0;
	return if is_administer($user, $blog_id);

	# 承認者選択のパラメータがあればhiddenに追加
	my $checkers = $q->{param}->{'checkrelease_checkers[]'} || [];
	if ( @$checkers ) {
		foreach ( @$checkers ) {
			push @$data, {
				data_name => 'checkrelease_checkers[]',
				data_value => $_
			};
		}
	}
	else {
		# 未選択状態の場合
		push @$data, {
			data_name => 'checkrelease_checkers[]',
			data_value => ''
		};
	}

	# リヴィジョンのパラメータを追加
	foreach ( qw/r crr/ ) {
		my $key = $_;
		if ( $_ eq 'r' ) {
			$key = 'pre_r';
		}
		push @$data, {
			data_name => $key,
			data_value => $app->param($_)
		};
	}

	# リビジョンのプレビューでボタンのラベルを「保存する」に補正
	$entry->status( MT::Entry::HOLD() ) if defined $q->param('crr');
}

sub _hdlr_cms_post_save {
	my ($cb, $app, $obj, $obj_org) = @_;
	return if unable_plugin();

	my $q = $app->param;

	my $plugin = $app->component('CheckRelease');
	my $blog_id = $q->param('blog_id') || 0;
	my $scope = "blog:$blog_id";
	my $config = get_config_hash($scope);

	my $user = $app->user;

	# 承認者の場合処理不要
	return if is_administer($user, $blog_id);
	my %param_hash = $app->param_hash;

	# 承認者を取得
	my $select_checker = ( defined($config->{select_checker})
		&& $config->{select_checker} ) ? 1 : 0;
	my $terms;
	if ( $select_checker ) {
		$terms->{id} = $q->{param}->{'checkrelease_checkers[]'} || [];
	}

	my $checkers = get_checkers( defined($terms->{id}) ? $terms : undef );

	# 承認申請情報を保存
	my @ids = map { $_->id } @$checkers;
	my $ids = @ids ? join(',', @ids) : '';
	my $check_status = $q->param('check_status');
	if (my $save_draft = $q->param('save_draft') == 1) {
		$ids = undef;
	}

	# 新規作成または承認申請の場合
	if ( $param_hash{status} == MT::Entry::RELEASE()
		|| $param_hash{status} == MT::Entry::FUTURE() ) {
		$obj->check_requested_authors( '' );
		$obj->check_status( APPROVAL() );
	}
	else {
		# ライターが下書きした場合は承認中にしない
		if ( $ids ) {
			$obj->check_requested_authors( $ids );
			$obj->check_status( SUBSCRIBED() );
		}
	}
	my $blog = $app->model('blog')->load($blog_id);
	$obj->checked_on(
		format_ts(
			'%Y-%m-%d %H:%M:%S',
			epoch2ts($blog, time()), $blog, 'en', 0) );
	$obj->save()
		or return $cb->error( $obj->errstr );

	# 承認者に投稿通知
	send_notice_mail( $app, $obj ) if $ids;

	# 編集不可の場合記事一覧へリダイレクト
	if ( defined($config->{lock_entry}) && $config->{lock_entry}
		&& $obj->check_status == SUBSCRIBED() ) {
		MT::Revisable::mt_postsave_obj(undef, $app, $obj, $obj_org);
		my $arg;
		$arg->{_type} = $app->param('_type');
		$arg->{blog_id} = $blog_id;
		my $goto = list_mode($obj->datasource);
		my $list_uri = $app->mt_uri( mode => $goto, args => $arg );
		return do_redirect($app, $list_uri);
	}
}

sub list_mode {
	my $class = shift;
	return MT->version_number >= 5.1 ? 'list' : "list_$class";
}

sub _hdlr_template_source_entry_table {
	my ($eh, $app, $tmpl) = @_;
	return $$tmpl if unable_plugin();

	my $user = $app->user;
	my $blog_id = $app->param('blog_id');
	return $$tmpl if is_administer($user, $blog_id);

	# 記事一覧以外の画面では処理しない
	my $mode = $app->param('__mode') || '';
	return $$tmpl unless $mode eq 'list_entry';

	# 記事のステータスアイコン部分に承認系アイコンを表示
	my $icon_old = '<mt:if name="status_spam">\s+?<a href=.+?<mt:if[^>]+?>.+?</mt:if>.+?<mt:if[^>]+?>.+?</mt:if>.+?</a>\s*?</mt:if>';
	my $icon_new = '<mt:if name="check_status_subscribed"><img src="<mt:StaticWebPath />images/status_icons/warning.gif" width="9" height="9" /></mt:if><mt:if name="check_status_return"><img src="<mt:StaticWebPath />images/status_icons/share.gif" width="9" height="9" /></mt:if>';

	$$tmpl =~ s!($icon_old)!$1\n$icon_new!mgs;

	# リビジョン作成リンクを表示
	my $rev_old = '<th class="category">';
	my $rev_new = '<th style="width:12%;"><__trans_section component="CheckRelease"><__trans phrase="Revision" /></__trans_section></th>';
	$$tmpl =~ s!$rev_old!$rev_new\n$rev_old!m;

	my $old = '<td class="category">';
	my $new = '<td>' .list_html_tmpl() .'</td>';

	$$tmpl =~ s!$old!$new\n$old!m;
	$$tmpl;
}

sub _hdlr_template_param_list_entry {
	my ($cb, $app, $param, $tmpl) = @_;
	return if unable_plugin();

	my $plugin = $app->component('CheckRelease');

	# 承認者でなければ処理不要
	my $user = $app->user;
	my $blog_id = $app->param('blog_id');
	return if is_administer($user, $blog_id);

	# 記事一覧以外の画面では処理しない
	my $mode = $app->param('__mode') || '';
	my $type = $app->param('_type') || 'entry';
	return $$tmpl unless $mode eq list_mode($type);

	my $class = $app->param('_type') || 'entry';

	# 記事ロックの設定を取得
	my $scope = 'blog:' .$app->param('blog_id');
	my $lock_entry = get_config_value('lock_entry', $scope) || 0;

	if ( MT->version_number < 5.1 ) {
		# ユーザーの権限を取得
		my $perms = $user->blog_perm($blog_id);

		my $can_edit_all_posts = $perms->can_edit_all_posts() || 0;
		my $can_publish_post = $perms->can_publish_post || 0;

		# 承認中の記事のアイコンの表示切替
		require MT::Serialize;
		foreach my $entry ( @{ $param->{entry_table}[0]{object_loop} } ) {
			$entry = get_entry_status($app, $user, $entry, $class, $lock_entry, $can_edit_all_posts, $can_publish_post);
		}
	}
	# 記事ロックフラグがパラメータで渡されたらonにする
	$param->{lock_entry} = 1 if $app->param('lock_entry');
}

sub get_entry_status {
	my ($app, $user, $entry, $class, $lock_entry, $can_edit_all_posts, $can_publish_post, $obj) = @_;

	if ( defined($obj) && ref($obj) ) {
		my $cols = $obj->column_names;
		foreach my $c ( @$cols ) {
			$entry->{$c} = $obj->$c;
		}
	}

	if ( defined($entry->{check_status}) && $entry->{check_status} ) {
		# 記事ロックの場合編集不可
		delete $entry->{status_Draft};
		if ( $entry->{check_status} == RETURN() ) {
			$entry->{check_status_return} = 1;
		} elsif ( $entry->{check_status} == SUBSCRIBED() ) {
			$entry->{has_edit_access} = $lock_entry ? 0 : 1;
			$entry->{check_status_subscribed} = 1;
		}
	}

	# 記事編集禁止の場合
	my $is_released = 0;
	if ( $entry->{author_id} == $user->id
		&& ($entry->{status} == MT::Entry::RELEASE()
			|| $entry->{status} == MT::Entry::FUTURE()) ) {
		++$is_released;
		$entry->{rev_has_edit_access} = 0;
		$entry->{rev_has_new_access} = 1;
		$entry->{has_edit_access} = 0 if $lock_entry;
	}


	# 最新リビジョンの取得
	my $rev_pkg = $app->model($class .':revision');
	my $terms;
	$terms->{entry_id} = $entry->{id};
	my $args;
	$args->{limit} = 1;
	$args->{'sort'} = 'id';
	$args->{direction} = 'descend';
	my @rev = $rev_pkg->load($terms, $args);
	my $rev_number = 0;
	if ( @rev ) {
		$rev_number = $rev[0]->rev_number;

		# リビジョンの公開状態を取得
		my $rev_entry = MT::Serialize->unserialize($rev[0]->$class);
		if ( $entry->{current_revision} < $rev_number ) {

			# リビジョンの承認状態を取得
			my $check_status = $$rev_entry->{check_status} || 0;
			my $status = $$rev_entry->{status} || 0;
			my $is_approved = $status == MT::Entry::RELEASE()
				|| $status == MT::Entry::FUTURE()
				|| $status == MT::Entry::JUNK()
				|| $status == 6 ? 1 : 0; # 6=MT::Entry::UNPUBLISH() compatibility under 6

			#$entry->{rev_has_edit_access} = 1;
			if ( !$is_approved && $check_status == SUBSCRIBED() ) {
				$entry->{has_edit_access} = $lock_entry ? 0 : 1;
				$entry->{rev_has_new_access} = 0;
				$entry->{rev_has_edit_access} = !$lock_entry;
				$entry->{rev_check_status_subscribed} = 1;
			} elsif ( !$is_approved && $check_status == RETURN() ) {
				$entry->{rev_check_status_return} = 1;
				$entry->{rev_has_edit_access} = 1;
			} elsif ( $status == MT::Entry::HOLD() ) {
				$entry->{rev_status_draft} = 1;
				$entry->{rev_has_edit_access} = 1;
			} elsif ( $status == MT::Entry::RELEASE() ) {
				$entry->{rev_status_publish} = 1;
			} elsif ( $status == MT::Entry::FUTURE() ) {
				$entry->{rev_status_future} = 1;

				# 公開日指定されたリビジョンの編集を許可すべきかどうか
				$entry->{rev_has_edit_access} = 0;
			} elsif ( $status == 6 ) { # 6=MT::Entry::UNPUBLISH() compatibility under 6
				$entry->{rev_status_unpublish} = 1;

				# 公開日非公開を指定されたリビジョンの編集を許可すべきかどうか
				$entry->{rev_has_edit_access} = 0;
			}
		}
		if ( $rev_number == $entry->{current_revision} ) {
			$entry->{is_latest} = 1;
		}

		# 全記事編集権限がなければ自分の記事以外は編集不可
		unless ( $can_edit_all_posts ) {
			if ( $entry->{current_revision} < $$rev_entry->{current_revision}
				&& $user->id != $$rev_entry->{created_by} ) {
				$entry->{rev_has_new_access} = 0;
				$entry->{rev_has_edit_access} = 0;
			}
		}
	}
	$entry->{rev_number} = $rev_number;

	# 全記事編集権限または自分の記事以外は編集不可
	unless ( $can_edit_all_posts ) {
		unless ( $user->id != $entry->{created_by} ) {
			$entry->{rev_has_edit_access} = 1
				unless $lock_entry;
		}
		if ( $entry->{status} != MT::Entry::HOLD() ) {
			$entry->{has_edit_access} = 0;
		} else {
			$entry->{_status_Draft} = 1;
		}
	}
	$entry;
}

sub _hdlr_cms_pre_save_entry {
	my ($cb, $app, $obj, $obj_orig) = @_;
	return 1 if unable_plugin();

	# 変更メモはリビジョンに残す
	if ( $app->param('revision-note') ) {
		$obj->{changed_revisioned_cols} = ['id'];
	}

	# 承認者で無ければ処理不要
	my $user = $app->user;
    my $blog_id = $app->param('blog_id');
    return 1 if is_administer($user, $blog_id);

	# 新規作成の場合処理不要
	return 1 unless $app->param('id');

	# リビジョンでなければ処理不要
	my $q = $app->param;
	my $crr = $q->param('crr') || 0;
	return 1 unless $crr;

	# 最新リビジョンを取得
	my $rev_obj = load_latest_revision($app, $obj->datasource, $obj->id);
	my $r = $q->param('r') || 0;

	# 公開状態を下書きに変更
	$app->param('status', MT::Entry::HOLD());
	my $status = $app->param('status');

	# MT5.1の権限バリデーション回避
	#if ( MT->version_number >= 5.1 && $r != 0 ) {
	#	$status = $app->param('original_status');
	#}

	# 承認者の選択
	my $config = get_config_hash();
	my $terms;
	if ( defined($config->{select_checker}) && $config->{select_checker} ) {
		$terms->{id} = $q->{param}->{'checkrelease_checkers[]'} || '';
	}

	my $checkers = [];

	# 承認申請情報を保存
	my @ids;
	if ( defined($terms->{id}) ) {
		$checkers = $terms->{id} || [];
	}
	else {
		my $candidates = get_checkers() || [];
		$checkers = [ map { $_->id } @$candidates ];
	}

	my $ids = join ',', @$checkers;
	$obj->check_requested_authors( $ids );

	my $save_draft = $app->param('save_draft');
	$ids = '' if $save_draft == 1;

	# 承認状態の変更
	my $blog = $obj->blog;
	my $check_status = $app->param('check_status');
	$obj->check_status( SUBSCRIBED() ) if $ids;
	$obj->status( $status );
	$obj->checked_on(
		format_ts(
			'%Y-%m-%d %H:%M:%S',
			epoch2ts($blog, time()), $blog, 'en', 0) );
	$obj->modified_by( $user->id );
	$obj->modified_on( format_ts(
		'%Y-%m-%d %H:%M:%S', epoch2ts($blog, time()), $blog) );

	# 最新リビジョンを作成
	my $new_revision = ($r == 0) ? 1 : 0;
	if ( $r == 0 || (ref($rev_obj) && $r && $r < $rev_obj->rev_number) ) {
		$obj->{changed_revisioned_cols} = [];
		my $latest_rev_number = ref($rev_obj) ? $rev_obj->rev_number : 0;
		my $rev_num = ($r == 0) ? $latest_rev_number : 0;
		local $obj->{column_values}->{current_revision} = $rev_num;
		$r = $obj->save_revision();
		$rev_obj = load_latest_revision($app, $obj->datasource, $obj->id);
	}
	if ( MT->version_number >= 5.1 && !defined $q->param('status') ) {
		$status = MT::Entry::HOLD();
		$obj->status( $status );
	}

	# リビジョンハッシュ化前のコールバック(MoreCustomFields対策)
	$app->run_callbacks('checkrelease_pre_pack_revision.entry', $app, $obj, $obj_orig);

	# リビジョンのハッシュを取得
	my $packed_obj = $obj->pack_revision();
	$packed_obj->{created_by} = $user->id if $new_revision;

	# カスタムフィールドの値を上書き
	my $cf_meta = cf_meta($app, $obj);
    foreach my $key (keys %$cf_meta) {
    	$packed_obj->{"field.$key"} = $cf_meta->{$key};
    }

    my $type = $app->param('_type') || 'entry';
    my $class = $app->model($type)
    	or return $app->errtrans('Invalid parameter');
    my $cat_class = $app->model( $class->container_type );

    # アイテムとの関連を保存
    my @packed_objassets;
    require MT::Asset;
    require MT::ObjectAsset;
    my $include_asset_ids = $app->param('include_asset_ids');
    my @asset_ids = split(',', $include_asset_ids);
    my $obj_assets = ();
    my @obj_assets = MT::ObjectAsset->load({ object_ds => 'entry', object_id => $obj->id });
    foreach my $obj_asset (@obj_assets) {
        my $asset_id = $obj_asset->asset_id;
        $obj_assets->{$asset_id} = 1;
    }
    my $seen = ();
    foreach my $asset_id (@asset_ids) {
        my $obj_asset = MT::ObjectAsset->load({ asset_id => $asset_id, object_ds => 'entry', object_id => $obj->id });
        unless ($obj_asset) {
            my $obj_asset = new MT::ObjectAsset;
            $obj_asset->blog_id($blog_id);
            $obj_asset->asset_id($asset_id);
            $obj_asset->object_ds('entry');
            $obj_asset->object_id($obj->id);
            my $packed = pack_revision($obj_asset);
            push @packed_objassets, $packed;
        }
    }
    $packed_obj->{objectasset} = \@packed_objassets;

    # カテゴリの関連を保存
    my ( $cat_id, @add_cat ) = split /\s*,\s*/,
      ( $app->param('category_ids') || '' );

    my @packed_places;
    require MT::Placement;
    my $place =
      MT::Placement->load( { entry_id => $obj->id, is_primary => 1 } );
    unless ($place) {
        $place = MT::Placement->new;
        $place->entry_id( $obj->id );
        $place->blog_id( $obj->blog_id );
        $place->is_primary(1);
    }
    $place->category_id($cat_id);
    push @packed_places, pack_revision($place);

    # save secondary placements...
    for my $cat_id (@add_cat) {
        my $cat = $cat_class->load($cat_id);

        # blog_id sanity check
        next if !$cat || $cat->blog_id != $obj->blog_id;

        my $place = MT::Placement->new;
        $place->entry_id( $obj->id );
        $place->blog_id( $obj->blog_id );
        $place->is_primary(0);
        $place->category_id($cat_id);
        push @packed_places, pack_revision($place);
    }
    $packed_obj->{placement} = \@packed_places;

	# タグの保存
	$packed_obj->{tags} = $app->param('tags') || '';

	$obj->gather_changed_cols($obj_orig, $app);
	my $changed_cols = $obj->{changed_revisioned_cols} || [];
	my $datasource = $obj->datasource;
	my $obj_id = $datasource .'_id';

	require MT::Serialize;
	$rev_obj->$datasource( MT::Serialize->serialize(\$packed_obj) );
	$rev_obj->changed( join ',', @$changed_cols );
	$rev_obj->description( $app->param('revision-note') );
	$rev_obj->save()
		or return $cb->erorr( $rev_obj->errstr );

	# 日時指定公開の場合は日時指定キューを更新
	update_future_que($app, $rev_obj, $datasource);

	# 承認者がいれば
	if ( $ids ) {
		# 承認者に投稿通知
		my $new_revision = ($q->param('r') == 0) ? 1 : 0;
		$q->param('r', $rev_obj->rev_number);
		send_notice_mail( $app, $obj, undef, $new_revision );
	}

	# 自動保存のセッション情報を削除
    my $sess_obj = $app->autosave_session_obj;
    $sess_obj->remove if $sess_obj;

	# リダイレクト
	my $args;
	$args->{r} = $rev_obj->rev_number;
	foreach ( qw/blog_id id _type crr/ ) {
		$args->{$_} = $q->param($_);
	}
	$args->{saved_changes} = 1;

	# 記事編集禁止の場合
	my $lock_entry = ( defined($config->{lock_entry})
		&& $config->{lock_entry} && $obj->check_status == SUBSCRIBED() ) ? 1 : 0;
	my $url = $app->mt_uri(
		mode => $lock_entry ? list_mode($type) : 'view',
		args => $args );
	return do_redirect($app, $url);
}

sub do_redirect {
	my ($app, $url) = @_;
	my $q = $app->param;
	my $qr = ref $q;
	if ( $ENV{MOD_PERL} ) {
	    $app->{apache}->header_out( Location => $url );
	    $app->response_code( Apache::Constants::REDIRECT() );
	    $app->send_http_header;
	}
	elsif ( $qr =~ /::PSGI/i ) {
		$app->redirect($url);
		return 0;
	}
	else {
	    $app->print(
	        $q->redirect( -uri => $url, %{ $app->{cgi_headers} } ) );
	}
	exit;
}

sub pack_revision {
	my $obj = shift;
	my $class = ref $obj || $obj;
	my $values = $obj->column_values;
	my $meta_values = $obj->meta;
	foreach my $key (keys %$meta_values) {
		next if $key eq 'current_version';
		$values->{$key} = $meta_values->{$key};
	}
	return $values;
}

sub _hdlr_pre_run {
	my ($cb, $app) = @_;
	return 1 if unable_plugin();
	return 1 unless ref($app) eq 'MT::App::CMS';
	my $q = $app->param;

	my $class = $q->param('_type') || '';
	return 1 unless $class eq 'entry';

	my $blog_id = $q->param('blog_id') || 0;
	return 1 unless $blog_id;

	# 管理者の場合処理不要
	my $user = $app->user;
	return 1 if is_administer($user, $blog_id);

	# 記事ロック設定かどうか
	my $scope = "blog:$blog_id";
	my $lock_entry = get_config_value('lock_entry', $scope);

	require MT::Serialize;
	my $plugin = $app->component('CheckRelease');
	my $id = $q->param('id');
	my $r = defined($q->param('r')) ?  $q->param('r') : '';
	my $crr = $q->param('crr') || '';
	# エラー発生の有無を評価
	my $reedit = defined($app->param('reedit')) ? 1 : 0;

	# 記事ロック設定の場合
	my $mode = $app->mode;
	my $goto = undef;
	my $params;
	if ( $mode eq 'view' ) {
		my $entry = {};
		my $rev_obj = load_latest_revision($app, $class, $id);
		if ( $r ne '0' && $crr && ref($rev_obj) ) {
			my $packed = MT::Serialize->unserialize( $rev_obj->$class ) || {};
			$entry = $$packed if defined($$packed->{check_status});
		}

		unless ( %$entry ) {
			my $pkg = $app->model($class);
			my $obj = $id ? $pkg->load($id) : $pkg->new;
			my $columns = $obj->column_names;
			%$entry = map { $_ => $obj->$_ } @$columns;
		}
		$params->{blog_id} = $blog_id;
		$params->{_type} = $class;

		# 記事編集画面の場合
		if ( !$crr && $id
			&& (
				( $entry->{status} == MT::Entry::RELEASE()
				|| $entry->{status} == MT::Entry::FUTURE() )
				|| (
					$lock_entry
					&& defined($entry->{check_status})
					&& $entry->{check_status} == SUBSCRIBED() )
				) ) {
			$goto = list_mode($class);
		}
		elsif ( $crr
			&& (
				$entry->{status} == MT::Entry::FUTURE()
				|| (
					$lock_entry
					&& $entry->{check_status} == SUBSCRIBED()
				) ) ) {
				# リビジョンの編集の場合
			$goto = list_mode($class);
		}

		# リビジョン新規作成又はプレビューから戻った場合はリダイレクトしない
		$goto = undef if $reedit || $r eq '0';

		# 宛先がある場合はリダイレクト
		if ( defined $goto ) {
			my $uri = $app->mt_uri( mode => $goto, args => $params);
			return $app->redirect($uri);
		}
	}
	# MT5.1の権限バリデーション回避
	if ( MT->version_number >= 5.1 && $id ) {
		$app->param('original_status', $app->param('status'));
		my $entry = $app->model('entry')->load( $id );
		$app->param('status', $entry->status);
	}
	return 1 unless $r && $crr;

	# リビジョン保存時に公開権限を一時的に付与しないと権限エラー
	if ( $mode eq "save_$class" ) {
		return unless ref $user;
		my $perms = $app->permissions;
		$perms->set_these_permissions('publish_post', 'manage_pages');
		$app->permissions( $perms );

	}
	return 1 unless $mode eq 'view';

	# プレビューでない場合
	unless ( $reedit ) {
		# リビジョンを取得
		my $rev_obj = load_latest_revision($app, $class, $id);
		my $packed = $rev_obj->$class;

		$packed = MT::Serialize->unserialize($packed);

		# アイテムのパラメータを擬似復元
		my $packed_objassets = defined($$packed->{objectasset})
			? $$packed->{objectasset}
			: [];
		if ( @$packed_objassets ) {
			$q->param('include_asset_ids',
				join(',', map { $_->{asset_id} } @$packed_objassets) );
		}

		# カテゴリのパラメータを擬似復元
		my $packed_placement = defined($$packed->{placement})
			? $$packed->{placement}
			: [];
		if ( @$packed_placement && $q->param('crr') ) {
			my @cat_ids = map { $_->{category_id} } grep { defined $_->{category_id} } @$packed_placement;
			$q->param('category_ids',  join(',', @cat_ids )) if @cat_ids;
		}
	}
}

sub _hdlr_template_source_list_entry {
	my ($cb, $app, $tmpl) = @_;
	return if unable_plugin();

	my $user = $app->user;
	my $blog_id = $app->param('blog_id');
	return if is_administer($user, $blog_id);

	# 記事一覧以外の画面では処理しない
	my $mode = $app->param('__mode') || '';
	my $type = $app->param('_type') || 'entry';
	return $$tmpl unless $mode eq list_mode($type);

	my $scope = "blog:$blog_id";
	my $lock_entry = get_config_value('lock_entry', $scope);

	# 記事ロック中に編集画面等に直接アクセスした場合のエラーメッセージ
	if ( $lock_entry ) {
		my $status_deleted = MT->version_number < 5.1
			? '<mt:if name="saved_deleted">'
			: '<mt:if name="request\.saved_deleted">';

		my $status_lock_entry = q{
    <mt:if name="lock_entry">
        <mtapp:statusmsg
            id="lock-entry"
            class="error"
            rebuild=0>
            <__trans_section component="CheckRelease">
            <mt:if name="object_type" eq="entry">
                <__trans phrase="The entry is subscribed and locked.">
            <mt:else>
                <__trans phrase="The page is subscribed and locked.">
            </mt:if>
        </mtapp:statusmsg>
    </mt:if>
		};
		$$tmpl =~ s!($status_deleted)!$status_lock_entry\n$1!mi;
	}
}

sub _hdlr_cms_save_filter_entry {
	my ($cb, $app) = @_;
	return 1 if unable_plugin();

	my $q = $app->param;
	my $blog_id = $q->param('blog_id') || 0;
	my $user = $app->user;
	return 1 if is_administer($user, $blog_id);

	my $config = get_config_hash();
	my $plugin = $app->component('CheckRelease');

	my $save_draft = $app->param('save_draft');

	# 承認者の選択必須の場合
	my $terms;
	if ( defined($config->{select_checker}) && $config->{select_checker} && $save_draft == 0 ) {
		$terms->{id} = $q->{param}->{'checkrelease_checkers[]'} || [];
		if ( !@{$terms->{id}} && (
			!defined($config->{check_required})
			|| (
				defined($config->{check_required})
				&& $config->{check_required}
				)
			) ) {

			# リビジョン更新でエラーが発生した場合
			my $errstr = $plugin->translate('Auchenticator is required.');
			if ( $app->param('crr') ) {
				my %params = $app->param_hash;
				$params{error} = $errstr;
				$params{reedit} = 'reedit';
				$params{dirty} = 0;
				$params{pre_r} = $params{r} if defined $params{r};
				delete $params{r};
				delete $params{return_args};

				my $type = delete $params{_type};
				my ( $token, $salt ) = set_param_session($app->config('CRRedirectURLExpires'), \%params);
				my %args = ( _type => $type, cr_token => $token, cr_salt => $salt );

				my $uri = $app->mt_uri( mode => 'view', args => \%args );
				return do_redirect($app, $uri);
			}

			return $cb->error( $errstr );
		}
	}
	1;
}
1;
